import bpy
import bmesh
import sys
import os
import pathlib
from bpy_extras import object_utils

class AddLowPolyLightning(bpy.types.Operator):
    bl_label = "New LowPoly Lightning"
    bl_idname = 'lightning.lowpoly'
    bl_options = {"REGISTER", "UNDO"}
    
    @classmethod
    def description(cls, context, properties):
        return "Adds a new 'LowPoly Lightning' at the position of the 3D cursor"
    
    def execute(self, context):        

        scene = bpy.context.scene
        savePPoint = scene.tool_settings.transform_pivot_point
        scene.tool_settings.transform_pivot_point = 'MEDIAN_POINT'
        cursorLoc = bpy.context.scene.cursor.location
        
        ui_Props = bpy.context.scene.lowPolyL_props
        lookAtObject = bpy.context.scene.lowPoly_rand
        lookAtChecked = ui_Props.lowPolyL_lookAtCamera


        #Check for already existing object with name--------------------------------------------------------------------------------------------------------
        def setUniqueName(newName):
            if bpy.data.objects.get(newName) is not None:
                checkName = newName
                counter = 1
                while bpy.data.objects.get(checkName) is not None:
                    counter += 1
                    checkName = newName + "_" + str(counter)
                return checkName
            else:
                return newName
            
            
            
        #Create Driver--------------------------------------------------------------------------------------------------------------------------------------
        def add_driver(source, prop, dataPath, name = '',func = ''):
            d = source.driver_add( prop ).driver
            v = d.variables.new()
            v.name                 = name
            v.targets[0].id        = source
            v.targets[0].data_path = dataPath
            d.expression = func



        #Add LightningMesh----------------------------------------------------------------------------------------------------------------------------------
        mesh = bpy.ops.mesh.primitive_plane_add(size=2, enter_editmode=False, align='WORLD', location=(0, 0, 0), scale=(1, 1, 1))
        bpy.ops.object.mode_set(mode='EDIT')
        lightningMeshName = setUniqueName("LightningMesh")
        bpy.context.object.name = lightningMeshName
        lightningMesh = bpy.data.objects[lightningMeshName]
        
        me = lightningMesh.data
        bm = bmesh.from_edit_mesh(me)
        bpy.ops.mesh.select_all(action='DESELECT')
        bm.verts.ensure_lookup_table()
        bm.verts[0].select = True
        bm.verts[1].select = True
        
        bpy.ops.transform.translate(value=(1, 0, 0), orient_type='LOCAL', orient_matrix=((-3.61999e-06, -1, 0), (1, -3.61999e-06, 0), (0, 0, 1)), orient_matrix_type='LOCAL', constraint_axis=(False, True, False), mirror=True, use_proportional_edit=False, proportional_edit_falloff='SMOOTH', proportional_size=1, use_proportional_connected=False, use_proportional_projected=False)

        
        bpy.ops.object.editmode_toggle()
        bpy.context.object.show_wire = True

        bpy.ops.transform.rotate(value=1.5708, orient_axis='Z', orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')



        #Add LightningRoot----------------------------------------------------------------------------------------------------------------------------------
        bpy.ops.object.empty_add(type='CIRCLE', align='WORLD', location=lightningMesh.location, scale=(1, 1, 1))
        lightningRootName = setUniqueName("SimpleLightning_Root")
        bpy.context.object.name = lightningRootName
        lightningRoot = bpy.data.objects[lightningRootName]
        bpy.ops.transform.rotate(value=1.5708, orient_axis='Z', orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.transform.resize(value=(0.3, 0.3, 0.3), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))

        #Make Root Parent of Mesh
        lightningMesh.parent = lightningRoot
        lightningMesh.matrix_parent_inverse = lightningRoot.matrix_world.inverted()
        
        
        
        #Add Lightning LookAt Target if custom LookAt Target is unchecked-----------------------------------------------------------------------------------
        if lookAtChecked == False:
            bpy.ops.object.empty_add(type='PLAIN_AXES', align='WORLD', location=(0, 0, 0), scale=(1, 1, 1))
            bpy.ops.transform.resize(value=(0.25, 0.25, 0.25), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
            bpy.ops.transform.translate(value=(0, 0, 1.25), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
            controlNameLookAt = setUniqueName("Lightning_LookAtTarget")
            bpy.context.object.name = controlNameLookAt
            lightning_LookAt_CTRL = bpy.data.objects[controlNameLookAt]
            
            lightning_LookAt_CTRL.parent = lightningRoot
            lightning_LookAt_CTRL.matrix_parent_inverse = lightningRoot.matrix_world.inverted()
            bpy.ops.transform.translate(value=(-0.5, 0, 0), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))



        #Add Lightning Handles------------------------------------------------------------------------------------------------------------------------------
        #Add Hook 1
        bpy.ops.object.empty_add(type='SPHERE', align='WORLD', location=(0, 0, 0), scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.1, 0.1, 0.1), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.transform.translate(value=(-2, -0, -0), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        controlName1 = setUniqueName("LightningCTRL_1")
        bpy.context.object.name = controlName1
        lightningCTRL_1 = bpy.data.objects[controlName1]
        
        modifier1 = lightningMesh.modifiers.new("Hook1", 'HOOK')
        modifier1.object = lightningCTRL_1
        modifier1.vertex_indices_set([1, 0])
        modifier1.show_expanded = False
        
        bpy.ops.object.constraint_add(type='TRACK_TO')
        bpy.context.object.constraints["Track To"].track_axis = 'TRACK_X'
        bpy.context.object.constraints["Track To"].up_axis = 'UP_Z'
        
        bpy.ops.object.constraint_add(type='LOCKED_TRACK')
        
        if lookAtChecked == True:
            bpy.context.object.constraints["Locked Track"].target = lookAtObject
        else:
            bpy.context.object.constraints["Locked Track"].target = lightning_LookAt_CTRL
            
        bpy.context.object.constraints["Locked Track"].track_axis = 'TRACK_Z'
        bpy.context.object.constraints["Locked Track"].lock_axis = 'LOCK_X'
        
        lightningCTRL_1.parent = lightningRoot
        lightningCTRL_1.matrix_parent_inverse = lightningRoot.matrix_world.inverted()
        
        
        #Add Hook 2
        bpy.ops.object.empty_add(type='SPHERE', align='WORLD', location=(0, 0, 0), scale=(1, 1, 1))
        bpy.ops.transform.resize(value=(0.1, 0.1, 0.1), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        bpy.ops.transform.translate(value=(1, -0, -0), orient_type='GLOBAL', orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)))
        controlName2 = setUniqueName("LightningCTRL_2")
        bpy.context.object.name = controlName2
        lightningCTRL_2 = bpy.data.objects[controlName2]
        
        modifier2 = lightningMesh.modifiers.new("Hook1", 'HOOK')
        modifier2.object = lightningCTRL_2
        modifier2.vertex_indices_set([2, 3])
        modifier2.show_expanded = False
        
        bpy.ops.object.constraint_add(type='TRACK_TO')
        bpy.context.object.constraints["Track To"].track_axis = 'TRACK_NEGATIVE_X'
        bpy.context.object.constraints["Track To"].up_axis = 'UP_Z'
        bpy.context.object.constraints["Track To"].target = lightningCTRL_1
        
        bpy.ops.object.constraint_add(type='LOCKED_TRACK')
        
        if lookAtChecked == True:
            bpy.context.object.constraints["Locked Track"].target = lookAtObject
        else:
            bpy.context.object.constraints["Locked Track"].target = lightning_LookAt_CTRL
            
        bpy.context.object.constraints["Locked Track"].track_axis = 'TRACK_Z'
        bpy.context.object.constraints["Locked Track"].lock_axis = 'LOCK_X'
        
        bpy.ops.object.select_all(action='DESELECT')
        lightningCTRL_1.select_set(True)
        bpy.context.view_layer.objects.active = lightningCTRL_1
        bpy.context.object.constraints["Track To"].target = lightningCTRL_2
        
        lightningCTRL_2.parent = lightningRoot
        lightningCTRL_2.matrix_parent_inverse = lightningRoot.matrix_world.inverted()
        

        #Move All to Cursor   
        lightningRoot.matrix_world.translation = scene.cursor.location



        #Adding Effect Modifiers to LighningMesh------------------------------------------------------------------------------------------------------------


        #Create and/or Assign Material----------------------------------------------------------------------------------------------------------------------
        scene = bpy.context.scene
        py_file_path = os.path.dirname(os.path.abspath(__file__))
        mat_blend_file = os.path.join(py_file_path, 'Low Poly Lightning_Material.blend')
        inner_path = 'Material'
        material_name = 'Low Poly Lightning Material'
        full_path_to_mat = os.path.join(mat_blend_file, inner_path, material_name)
        appended_material = None
        
        for mat in bpy.data.materials:
            if 'Low Poly Lightning Material' == mat.name:
                appended_material = mat
                break
            else:
                bpy.ops.wm.append(filepath=full_path_to_mat, directory=os.path.join(mat_blend_file, inner_path), filename=material_name)
                appended_material = bpy.data.materials.get(material_name)
        
        lightningMesh.data.materials.append(appended_material)
 
        scene.tool_settings.transform_pivot_point = savePPoint   
        
        return {'FINISHED'} 
        
        
def register() :
    bpy.utils.register_class(AddLowPolyLightning)
 
def unregister() :
    bpy.utils.unregister_class(AddLowPolyLightning)